searchData={"content_type":"text/markdown","items":[{"doc":"Wireguardex is an Elixir library for configuring [WireGuard](https://wireguard.com) interfaces. It uses\n[Rust](https://rust-lang.org) NIFs for performance and safety.\n\nThis is the main module, providing the API for interface configuration and\nutilities such as key generation.","ref":"Wireguardex.html","title":"Wireguardex","type":"module"},{"doc":"Add a peer to a `Device` using a config for the peer. A config can be generated\nusing a `PeerConfigBuilder`.\n\nReturns `:ok` if successful. `{:error, error_info}` will be returned if adding\nthe peer to the device fails.","ref":"Wireguardex.html#add_peer/2","title":"Wireguardex.add_peer/2","type":"function"},{"doc":"Delete a `Device` by its interface name.\n\nReturns `:ok` if successful. `{:error, error_info}` will be returned if deleting\nthe device fails.","ref":"Wireguardex.html#delete_device/1","title":"Wireguardex.delete_device/1","type":"function"},{"doc":"Generates a random preshared key. It is returned as a base64 `string`.","ref":"Wireguardex.html#generate_preshared_key/0","title":"Wireguardex.generate_preshared_key/0","type":"function"},{"doc":"Generates a random private key. It is returned as a base64 `string`.","ref":"Wireguardex.html#generate_private_key/0","title":"Wireguardex.generate_private_key/0","type":"function"},{"doc":"Get a `Device` by its interface name.\n\nReturns `{:ok, Device}` if successful. `{:error, error_info}` will be returned\nif getting the device fails.","ref":"Wireguardex.html#get_device/1","title":"Wireguardex.get_device/1","type":"function"},{"doc":"Return a private key's public key as a base64 `string`.\n\nReturns `{:ok, public_key}` if successful. `{:error, error_info}` will be returned if\nif getting the public key fails.","ref":"Wireguardex.html#get_public_key/1","title":"Wireguardex.get_public_key/1","type":"function"},{"doc":"Get a list of interface names of WireGuard devices.\n\nReturns `{:ok, [...]}` if successful. `{:error, error_info}` will be returned\nif listing device interface names fails.","ref":"Wireguardex.html#list_devices/0","title":"Wireguardex.list_devices/0","type":"function"},{"doc":"Remove a peer from a `Device` by the peer's public key.\n\nReturns `:ok` if successful. `{:error, error_info}` will be returned if removing\nthe peer from the device fails.","ref":"Wireguardex.html#remove_peer/2","title":"Wireguardex.remove_peer/2","type":"function"},{"doc":"Set a `Device` by its interface name with a config. The config can be generated\nusing a `DeviceConfigBuilder`.\n\nNote if no device is present, a new one will be created for the given interface name.\n\nReturns `:ok` if successful. `{:error, error_info}` will be returned if setting\nthe device fails.","ref":"Wireguardex.html#set_device/2","title":"Wireguardex.set_device/2","type":"function"},{"doc":"","ref":"Wireguardex.html#t:device/0","title":"Wireguardex.device/0","type":"type"},{"doc":"","ref":"Wireguardex.html#t:device_config/0","title":"Wireguardex.device_config/0","type":"type"},{"doc":"","ref":"Wireguardex.html#t:key/0","title":"Wireguardex.key/0","type":"type"},{"doc":"","ref":"Wireguardex.html#t:name/0","title":"Wireguardex.name/0","type":"type"},{"doc":"","ref":"Wireguardex.html#t:peer_config/0","title":"Wireguardex.peer_config/0","type":"type"},{"doc":"","ref":"Wireguardex.html#t:peer_stats/0","title":"Wireguardex.peer_stats/0","type":"type"},{"doc":"`Device` represents all available information about a WireGuard device (interface).\n\nThis struct contains the current configuration of the interface and the current\nconfiguration and info for all of its peers.\n\n* `name` The interface name of the device.\n* `public_key` The public encryption key of the interface.\n* `private_key` The private encryption key of the interface.\n* `fwmark` The [fwmark](https://man7.org/linux/man-pages/man8/tc-fw.8.html) of\n  this interface.\n* `listen_port` The port to listen for incoming connections.\n* `peers` The list of all peers registered to this interface with their configs\n  and stats.\n* `linked_name` The associated \"real\" name of the interface.","ref":"Wireguardex.Device.html","title":"Wireguardex.Device","type":"module"},{"doc":"","ref":"Wireguardex.Device.html#t:t/0","title":"Wireguardex.Device.t/0","type":"type"},{"doc":"This module contains functions to create a `DeviceConfig` which itself\nrepresents a configuration that can be set on a Wireguard device (interface).\n\nIf an interface exists, the configuration is applied _on top_ of the existing\ninterface's settings. Missing settings are not overwritten or set to defaults.\n\nTo crate the default config use `device_config/0` then you can apply each\nfunction in this module to specify the details.","ref":"Wireguardex.DeviceConfigBuilder.html","title":"Wireguardex.DeviceConfigBuilder","type":"module"},{"doc":"iex> res=\n    ...> device_config()\n    ...> |> listen_port(1234)\n    ...> |> fwmark(11111)\n    ...> |> Wireguardex.set_device(\"wg1234\")\n    ...> Wireguardex.delete_device(\"wg1234\")\n    ...> res\n    :ok","ref":"Wireguardex.DeviceConfigBuilder.html#module-examples","title":"Examples - Wireguardex.DeviceConfigBuilder","type":"module"},{"doc":"Creates the default configuration to be configured with the provided functions.","ref":"Wireguardex.DeviceConfigBuilder.html#device_config/0","title":"Wireguardex.DeviceConfigBuilder.device_config/0","type":"function"},{"doc":"The [fwmark](https://man7.org/linux/man-pages/man8/tc-fw.8.html) to set on the\ninterface.","ref":"Wireguardex.DeviceConfigBuilder.html#fwmark/2","title":"Wireguardex.DeviceConfigBuilder.fwmark/2","type":"function"},{"doc":"The listening port for incoming connections to set on the interface.","ref":"Wireguardex.DeviceConfigBuilder.html#listen_port/2","title":"Wireguardex.DeviceConfigBuilder.listen_port/2","type":"function"},{"doc":"A list of peers with their own configurations to set on this interface.","ref":"Wireguardex.DeviceConfigBuilder.html#peers/2","title":"Wireguardex.DeviceConfigBuilder.peers/2","type":"function"},{"doc":"The private encryption key to set on the interface.","ref":"Wireguardex.DeviceConfigBuilder.html#private_key/2","title":"Wireguardex.DeviceConfigBuilder.private_key/2","type":"function"},{"doc":"The public encryption key to set on the interface.","ref":"Wireguardex.DeviceConfigBuilder.html#public_key/2","title":"Wireguardex.DeviceConfigBuilder.public_key/2","type":"function"},{"doc":"If true, replace existing peer configurations on the interface.","ref":"Wireguardex.DeviceConfigBuilder.html#replace_peers/2","title":"Wireguardex.DeviceConfigBuilder.replace_peers/2","type":"function"},{"doc":"This module contains functions to create a `PeerConfig`  which represents\na peer's configuration of persistent attributes. They do not change over\ntime and are part of the configuration of a device.\n\nTo create the default config use `peer_config/0` then you can apply each\nfunction in this module to specify the config.","ref":"Wireguardex.PeerConfigBuilder.html","title":"Wireguardex.PeerConfigBuilder","type":"module"},{"doc":"iex> {:ok, public_key} = Wireguardex.get_public_key(Wireguardex.generate_private_key())\n    ...> res =\n    ...> device_config()\n    ...> |> peers([\n    ...>   peer_config()\n    ...>   |> Wireguardex.PeerConfigBuilder.public_key(public_key)\n    ...>   |> endpoint(\"127.0.0.1:1234\")\n    ...>   |> preshared_key(Wireguardex.generate_preshared_key())\n    ...>   |> persistent_keepalive_interval(60)\n    ...>   |> allowed_ips([\"127.0.0.1/16\"])\n    ...> ])\n    ...> |> Wireguardex.set_device(\"wg12345\")\n    ...> Wireguardex.delete_device(\"wg12345\")\n    ...> res\n    :ok","ref":"Wireguardex.PeerConfigBuilder.html#module-examples","title":"Examples - Wireguardex.PeerConfigBuilder","type":"module"},{"doc":"The allowed ip addresses this peer is allowed to have.","ref":"Wireguardex.PeerConfigBuilder.html#allowed_ips/2","title":"Wireguardex.PeerConfigBuilder.allowed_ips/2","type":"function"},{"doc":"The endpoint this peer listens for connections on (`nil` means any).","ref":"Wireguardex.PeerConfigBuilder.html#endpoint/2","title":"Wireguardex.PeerConfigBuilder.endpoint/2","type":"function"},{"doc":"Creates the default peer configuration that you can then specify which each of\nthe provided functions.","ref":"Wireguardex.PeerConfigBuilder.html#peer_config/0","title":"Wireguardex.PeerConfigBuilder.peer_config/0","type":"function"},{"doc":"The interval for sending keepalive packets (`nil` means disabled).","ref":"Wireguardex.PeerConfigBuilder.html#persistent_keepalive_interval/2","title":"Wireguardex.PeerConfigBuilder.persistent_keepalive_interval/2","type":"function"},{"doc":"The preshared key available to both peers (`nil` means no PSK is used).","ref":"Wireguardex.PeerConfigBuilder.html#preshared_key/2","title":"Wireguardex.PeerConfigBuilder.preshared_key/2","type":"function"},{"doc":"The public key of the peer.","ref":"Wireguardex.PeerConfigBuilder.html#public_key/2","title":"Wireguardex.PeerConfigBuilder.public_key/2","type":"function"},{"doc":"`PeerInfo` represents all of the available information of a peer.\n\nThis struct is a simple pair of a peer's configuration and stats.\n\n* `config` The configuration belonging to this peer.\n* `stats` The current statistics of this peer.","ref":"Wireguardex.PeerInfo.html","title":"Wireguardex.PeerInfo","type":"module"},{"doc":"","ref":"Wireguardex.PeerInfo.html#t:t/0","title":"Wireguardex.PeerInfo.t/0","type":"type"},{"doc":"`PeerStats` represent a peer's statistics from the current session.\n\nThese are the attributes of a peer that will change over time; to get updated\nstats re-read the information from the interface.\n\n* `last_handshake_time` Timestamp of the last handshake/rekey with this peer.\n* `rx_bytes` Number of bytes received from this peer.\n* `tx_bytes` Number of bytes transmitted to this peer.","ref":"Wireguardex.PeerStats.html","title":"Wireguardex.PeerStats","type":"module"},{"doc":"","ref":"Wireguardex.PeerStats.html#t:t/0","title":"Wireguardex.PeerStats.t/0","type":"type"},{"doc":"# Wireguardex","ref":"readme.html","title":"Overview","type":"extras"},{"doc":"[![hex.pm](https://img.shields.io/hexpm/v/wireguardex.svg)](https://hex.pm/packages/wireguardex)\n[![hex.pm](https://img.shields.io/hexpm/dt/wireguardex.svg)](https://hex.pm/packages/wireguardex)\n[![hex.pm](https://img.shields.io/hexpm/l/wireguardex.svg)](https://hex.pm/packages/wireguardex)\n\nWireguardex is an Elixir library for configuring [WireGuard®](https://www.wireguard.com/) interfaces.\n\nIt is exposed as a native library via NIFs implemented in [Rust](https://rust-lang.org) using the [rustler](https://crates.io/crates/rustler) and [wireguard-control](https://docs.rs/wireguard-control/latest/wireguard_control/) crates.\n\nUsed by [Firezone](https://github.com/firezone/firezone) to manage WireGuard interfaces in Elixir.","ref":"readme.html#overview","title":"Overview - Overview","type":"extras"},{"doc":"Add `wireguardex` to your dependencies:\n\n```elixir\ndef deps do\n  [\n    {:wireguardex, \"~> 0.3\"}\n  ]\nend\n```\n\nThen you can just use wireguardex to manage your wireguard interfaces:\n\n```elixir\n# Imports for cleanliness\nimport Wireguardex.DeviceConfigBuilder\nimport Wireguardex.PeerConfigBuilder\nimport Wireguardex, only: [set_device: 2]\n\ninterface_name = \"wg0\"\nprivate_key = Wireguardex.generate_private_key()\n{:ok, public_key} = Wireguardex.get_public_key(private_key)\nlisten_port = 58210\nfwmark = 1234\n\n:ok =\n  device_config() #   private_key(private_key)\n  |> public_key(public_key)\n  |> listen_port(listen_port)\n  |> fwmark(fwmark)\n  |> set_device(interface_name) #   public_key(public_key)\n  |> preshared_key(Wireguardex.generate_preshared_key())\n  |> endpoint(\"127.0.0.1:1234\")\n  |> persistent_keepalive_interval(30)\n  |> allowed_ips([\"255.0.0.0/24\", \"127.0.0.0/16\"])\n\n# Add peer to existing device\n:ok = Wireguardex.add_peer(interface_name, peer)\n```\n\nAnd easily remove it afterwards using its public key:\n\n```elixir\n:ok = Wireguardex.remove_peer(interface_name, public_key)\n```\n\nTo get information on an existing device:\n\n```elixir\n{:ok, device} = Wireguardex.get_device(interface_name)\n```\n\nFinally to delete a device:\n\n```elixir\n:ok = Wireguardex.delete_device(interface_name)\n```","ref":"readme.html#getting-started","title":"Getting started - Overview","type":"extras"},{"doc":"The package can be installed by adding `wireguardex` to your list of dependencies\nin `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:wireguardex, \"~> 0.3\"}\n  ]\nend\n```\n\nWireguardex will try to download a precompiled NIF library. If you want to compile\nyour own NIF, you'll need to have Rust installed. The common option is to use\n[Rustup](https://rustup.rs/).\n\nTo force compilation you can set the environment variable `WIREGUARDNIF_BUILD`\nto `true` or `1`. Or you can set the application env to force the NIF to compile:\n\n```elixir\nconfig :rustler_precompiled, :force_build, wireguardex: true\n```\n\n#","ref":"readme.html#installation","title":"Installation - Overview","type":"extras"},{"doc":"This library creates and modifies network interfaces. If you'd like to run this library as a non-root user, we recommend adding the `CAP_NET_ADMIN` Linux capability to the Erlang VM executable:\n\n```sh\nsudo setcap 'cap_net_admin+eip'  /bin/beam.smp\n```\n\nIf you're using [asdf-vm](https://asdf-vm.com/) to manage dependencies you can do:\n\n```sh\nsudo setcap 'cap_net_admin+eip' $(ls -1 `asdf where erlang 24.3.4`/erts-*/bin/beam.smp)\n```\n\nThis can be handy for development and testing purposes.\n\n**Note**: This will also give `CAP_NET_ADMIN` to any other Erlang programs using this `beam.smp` executable. If this is undesired, consider using a dedicated Erlang installation or `beam.smp` executable for this library.","ref":"readme.html#note-about-privileges","title":"Note about privileges - Overview","type":"extras"},{"doc":"* Manage WireGuard interfaces\n* Doesn't require a WireGuard installation","ref":"readme.html#features","title":"Features - Overview","type":"extras"},{"doc":"Running the tests in this library will also require a Rust installation, as the NIF is compiled\nlocally before running the tests.\n\nFollow [these](https://www.rust-lang.org/learn/get-started) instructions to install Rust.\n\nThen you can run `mix test` as long as you have the [user privileges to create interfaces](#note-about-privileges).\n\n#","ref":"readme.html#tests","title":"Tests - Overview","type":"extras"},{"doc":"We use [pre-commit](https://pre-commit.com) to catch any static analysis issues before code is\ncommitted. Install with Homebrew: `brew install pre-commit` or pip: `pip install pre-commit`.","ref":"readme.html#pre-commit","title":"Pre-commit - Overview","type":"extras"},{"doc":"\"WireGuard\" and the \"WireGuard\" logo are registered trademarks of Jason A. Donenfeld.","ref":"readme.html#acknowledgments","title":"Acknowledgments - Overview","type":"extras"}]}